/* vim: set sw=4 ts=4 et: */
/*
 * This file is part of Live Wallpaper (livewp)
 * 
 * Copyright (C) 2010 Vlad Vasiliev
 * Copyright (C) 2010 Tanya Makova
 *       for the code
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
*/
/*******************************************************************************/
#include "livewp-settings.h"
/*******************************************************************************/
void lw_about(void){

    GtkWidget *window = NULL,
    *vbox = NULL,
    *label_about = NULL;
    window = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(window), _("About"));
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    gtk_window_set_modal(GTK_WINDOW(window), TRUE);
    vbox = gtk_vbox_new (FALSE, 5);
    label_about = gtk_label_new (_("Live Wallpaper\n Version 0.2 \n Copyright(c) 2010\n \
Tanya Makova\n Vlad Vasiliev\n \
Copyright(c) 2010 for design \n Vasya Bobrikov")); 
    gtk_box_pack_start (GTK_BOX (vbox), label_about, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox),
                                   vbox, TRUE, TRUE, 0);
    gtk_widget_show (label_about);
    gtk_widget_show (vbox);
    gtk_widget_show (window);
    gtk_dialog_run(GTK_DIALOG(window));

}
/*******************************************************************************/
GtkWidget *
create_theme_selector (void)
{
      GtkWidget *selector;

      selector = hildon_touch_selector_new_text ();

      hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), _("Berlin"));
      hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), _("Modern"));
      return selector;
}
/*******************************************************************************/
GtkWidget *
create_themes_button (gchar *theme){

    GtkWidget *button;
    GtkWidget *selector;

    selector = create_theme_selector();
    button = hildon_picker_button_new (HILDON_SIZE_AUTO, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
    hildon_button_set_title (HILDON_BUTTON (button), _("Theme"));
    hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (button),
                                                       HILDON_TOUCH_SELECTOR (selector));
    if (theme) {
        if (!strcmp(theme, "Berlin")){
            hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, 0);
            hildon_button_set_value(HILDON_BUTTON(button), _("Berlin"));
        }
        if (!strcmp(theme, "Modern")){
            hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, 1);
            hildon_button_set_value(HILDON_BUTTON(button), _("Modern"));
        }

    }
    return button;
}

/*******************************************************************************/
GtkWidget *
create_rich_animation_button (gboolean active)
{
    GtkWidget *button;
    button = hildon_check_button_new (HILDON_SIZE_AUTO);
    gtk_button_set_label (GTK_BUTTON (button), _("Rich Animation"));
    hildon_check_button_set_active(button, active);
    return button;
}

/*******************************************************************************/
void 
lw_settings(GtkWidget *widget, Animation_WallpaperPrivate *priv){
    gint result;
    GtkWidget *window = NULL;
    GtkWidget *save_button;
    GtkWidget *theme_button;
    GtkWidget *rich_animation_button;

    window = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(window), _("Live Wallpaper Settings"));
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    gtk_window_set_modal(GTK_WINDOW(window), TRUE);
    /* Create Theme button */
    theme_button = create_themes_button(priv->theme);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox),
                                   theme_button, TRUE, TRUE, 5);
    /* Create rich animation button */  
    rich_animation_button = create_rich_animation_button(priv->rich_animation);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox),
                                   rich_animation_button, TRUE, TRUE, 5);
    gtk_widget_show (theme_button);
    gtk_widget_show (rich_animation_button);
    gtk_widget_show (window);
    gtk_dialog_add_button(GTK_DIALOG(window), _("About"), GTK_RESPONSE_NO);
    save_button = gtk_dialog_add_button(GTK_DIALOG(window), _("Save"), GTK_RESPONSE_YES);
    result = gtk_dialog_run(GTK_DIALOG(window));

    switch(result){
        case GTK_RESPONSE_YES:
            if (hildon_button_get_value(HILDON_BUTTON (theme_button))){
                if (priv->theme)
                    g_free(priv->theme);
                if (!strcmp(hildon_button_get_value(HILDON_BUTTON (theme_button)), _("Berlin")))
                    priv->theme = g_strdup("Berlin");
                if (!strcmp(hildon_button_get_value(HILDON_BUTTON (theme_button)), _("Modern")))
                    priv->theme = g_strdup("Modern");
                if (hildon_check_button_get_active (rich_animation_button))
                    priv->rich_animation = TRUE;
                else
                    priv->rich_animation = FALSE;
            }
            save_config(priv);
            reload_scene(priv->desktop_plugin);
        break;
        default:
        case GTK_RESPONSE_OK:
        break;
        case GTK_RESPONSE_NO:
            gtk_widget_destroy(window);
            window = NULL;
            lw_about();
        break;
    }
    if (window)
        gtk_widget_destroy(window);
}
